typedef struct FILE_BUFFER file_buffer;
struct FILE_BUFFER {
    u8 *base;
    u64 capacity;
    u64 position;
};

file_buffer ReadWholeFile(memory_arena *arena, string filename) {
    file_buffer result = {0};

    void *handle = CreateFileA(StringToCString(filename), GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
    if (handle == INVALID_HANDLE_VALUE) {
        FatalWindowsError(GetLastError());
    } else {
        if (GetFileSizeEx(handle, (PLARGE_INTEGER)(&result.capacity)) == 0) {
            FatalWindowsError(GetLastError());
        } else if (result.capacity > 0) {
            result.base = GetMemoryArenaArray(arena, (*result.base), result.capacity);

            u64 read = 0;
            if (IsFalse(ReadFile(handle, result.base, (DWORD)result.capacity, (LPDWORD)&read, 0)) || read != result.capacity) {
                ReturnMemoryArenaArray(arena, *result.base, result.capacity);
                result = (file_buffer) {0};
            }
        }

        CloseHandle(handle);
    }

    return (result);
}

file_buffer FileBuffer(memory_arena *arena, u64 capacity) {
    file_buffer result = {0};

    result.capacity = capacity;
    result.base = GetMemoryArenaChunk(arena, capacity);

    return (result);
}

void * CreateFileToWrite(string filename) {
    void *result = 0;
    result = CreateFileA(StringToCString(filename), GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0);
    if (result == INVALID_HANDLE_VALUE) {
        FatalWindowsError(GetLastError());
    }
    return (result);
}

void WriteFileBufferToFile(void *file, file_buffer *buffer) {
    if (file) {
        WriteFile(file, &buffer->base[0], (DWORD)buffer->position, 0, 0);
    }
    buffer->position = 0;
}

void CloseFile(void *file) {
    if (file) {
        CloseHandle(file);
    }
}

void WriteStringToFileBuffer(void *file, file_buffer *buffer, string value) {
    u64 remaining = buffer->capacity - buffer->position;
    if (remaining < value.length) {
        WriteFileBufferToFile(file, buffer);
    }
    memcpy(&buffer->base[buffer->position], value.base, value.length);
    buffer->position += value.length;
}
