static u32 g_random_state_u32 = 1;
static u64 g_random_state_u64 = 1;

void InitRandom(void) {
    g_random_state_u32 = (u32)__rdtsc();
    g_random_state_u64 = __rdtsc();
}

u32 Random32(void) {
    /* Algorithm "xor" from p. 4 of Marsaglia, "Xorshift RNGs" */
    u32 result = g_random_state_u32;
    
    result ^= result << 13;
    result ^= result >> 17;
    result ^= result << 5;
    
    g_random_state_u32 = result;

    return (result);
}

u64 Random64(void) {
    u64 result = g_random_state_u64;
    
    result ^= result << 13;
    result ^= result >> 7;
    result ^= result << 17;
    
    g_random_state_u64 = result;

    return (result);
}

/* Gets random number from 0 to whatever the maximum is */
u32 RandomZeroRange32(u32 max) {
    Assert(max < U32_MAX);

    u32 result = Random32() % (max + 1);
    return (result);
}

/* Gets random number from 0 to whatever the maximum is */
u64 RandomZeroRange64(u64 max) {
    Assert(max < U64_MAX);

    u64 result = Random64() % (max + 1);
    return (result);
}

/* Gets random float from 0.0 to 1.0 */
f32 RandomFloat32() {
    f32 result = Random32() / (f32)U32_MAX;
    return (result);
}

/* Gets random float from 0.0 to 1.0 */
f64 RandomFloat64() {
    f64 result = Random64() / (f64)U64_MAX;
    return (result);
}
