// This is a separate compilation unit to link into your executable
// when building with MSVC and without the C runtime.
// It provides implementations for memcpy, memmove and memset.
// It also provides some internal variables used by the compiler
// that usually are provided by the C runtime.

// MSVC needs this variable to allow floating point usage without the CRT
int _fltused = 0x9875;

// Even though MSVC has a memcpy intrinsic it seems to need an implementation
#pragma function(memcpy)
void * memcpy(void *dst, const void *src, unsigned __int64 count) {
    char *destination = (char *)dst;
    const char *source = (char *)src;
    while(count--) {
        *destination++ = *source++;
    }
    return dst;
}

// Even though MSVC has a memmove intrinsic it seems to need an implementation
#pragma function(memmove)
void * memmove(void *dst, const void *src, unsigned __int64 count) {
    char *destination = (char *)dst;
    const char *source = (char *)src;
    
    if (destination < source) {
        while(count--) {
            *destination++ = *source++;
        }
    } else {
        char *last_dst = destination + (count - 1);
        const char *last_src = source + (count - 1);
        while (count--) {
            *last_dst-- = *last_src--;
        }
    }
    return dst;
}

// Even though MSVC has a memset intrinsic it seems to need an implementation
#pragma function(memset)
void * memset(void *dst, signed __int32 c, unsigned __int64 count) {
    char *destination = (char *)dst;
    while(count--) {
        *destination++ = (char)c;
    }
    return dst;
}
