// System includes
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <intrin.h>

// Danilib includes
#include "danilib/dani_base.h"
#include "danilib/dani_arena.h"
#include "danilib/dani_string.h"

#define DEFAULT_MEMORY_COMMIT_SIZE KiB(8)
#define DEFAULT_MEMORY_RESERVE_SIZE GiB(2)
#define DEFAULT_MEMORY_ALIGNMENT 32

// Project includes
#include "base/console.c"
#include "base/arguments.c"
#include "base/file.c"
#include "base/profiler.c"
#include "base/random.c"

// Danilib implementations
#define DANI_LIB_MEMORY_ARENA_IMPLEMENTATION
#define DANI_LIB_MEMORY_ARENA_PANIC
static void MemoryArenaPanic(void) {
    FatalWindowsError(GetLastError());
}

#include "danilib/dani_arena.h"
#undef DANI_LIB_MEMORY_ARENA_IMPLEMENTATION

#define DANI_LIB_STRING_IMPLEMENTATION
#include "danilib/dani_string.h"
#undef DANI_LIB_STRING_IMPLEMENTATION

#define EXIT_CODE_SUCCESS 0
#define EXIT_CODE_MISSING_ARGUMENT (0x20000000 + 2)
#define EXIT_CODE_INVALID_ARGUMENT (0x20000000 + 3)

#include "generator.c"
#include "benchmark.c"
#include "program.c"

void ProgramStart(void) {
    SetUpConsole();
    InitRandom();
    
    memory_arena *arena = MemoryArena(
        DEFAULT_MEMORY_RESERVE_SIZE,
        DEFAULT_MEMORY_COMMIT_SIZE,
        DEFAULT_MEMORY_ALIGNMENT);

    u32 exit_code;

    Unused(GetNextCommandLineArgument()); // Skip executable name
    string argument = GetNextCommandLineArgument();
    if (argument.length > 0) {
        if (IsStringEqual(argument, StringLiteral("generate"))) {
            exit_code = RunGenerator(arena);
        } else if (IsStringEqual(argument, StringLiteral("benchmark"))) {
            exit_code = RunBenchmark(arena);
        } else if (IsStringEqual(argument, StringLiteral("run"))) {
            exit_code = RunProgram(arena);
        } else {
            exit_code = EXIT_CODE_INVALID_ARGUMENT;
            string error = StringFormat(arena, "Invalid argument \"%.*s\" - Has to be \"generate\", \"benchmark\", or \"run\".\n", StringToFormatArgs(argument));
            WriteError(error);
        }
    } else {
        exit_code = EXIT_CODE_MISSING_ARGUMENT;
        string error = StringFormat(arena, "Missing argument - Has to be \"generate\", \"benchmark\", or \"run\".\n", StringToFormatArgs(argument));
        WriteError(error);
    }

    ExitProcess(exit_code); // TODO: Maybe use terminate process instead?
}
